# -*- encoding:utf-8 -*-
#
#------------------------------------------------------------------------------
# PyQGIS Application - Free GIS Software to view, query and analize geographic
# information.
# Based on Python bindings for Quantum GIS (Tim Sutton, Martin Dobias).
#
# Original sources Copyright (c) 2009 by Germn Carrillo
# geotux_tuxman@linuxmail.org
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#------------------------------------------------------------------------------

import imp, os, fileinput
from PyQt4.QtGui import QMenu

from dialogos.qgisInterface import QgisInterface

class Plugins():
    """ Class to load plugins to the main application """
    def __init__( self, visor, canvas ):
        self.qgisInterface = QgisInterface( visor, canvas )
        self.pyQgisApp = visor

        self.pyQgisApp.menuPlugins = None
        self.plugins = []
        self.cargarPlugins()

    def cargarPlugins( self ):
        """ Validate there is a Plugins folder and load the plugins to the app """
        dir_plugins = os.path.join( os.path.abspath( '' ), 'plugins' )

        if os.path.exists( dir_plugins ):
            for root, dirs, files in os.walk( dir_plugins ):
                bPlugIn = False

                if not dir_plugins == root: # Subfolders in folder Plugins
                    if '__init__.py' in files: # Filter plugins folders

                        # Verify that __init__.py file belongs to a plugin
                        for line in fileinput.input( os.path.join( root, '__init__.py' ) ):
                            linea = line.strip()
                            if linea == ("def classFactory(iface):") or linea == "def classFactory( iface ):":
                                bPlugIn = True
                                break
                        fileinput.close()

                        if bPlugIn:
                            if not self.pyQgisApp.menuPlugins:
                                self.crearMenuPlugins()
                                print "+----------------=== LOADED PLUGINS ===----------------+"

                            # Load the plugin
                            nombre_plugin = os.path.basename( root )
                            print ' * ',nombre_plugin,':',
                            f, filename, description = imp.find_module( nombre_plugin, [ 'plugins' ] )

                            try: # Load and initialize the plugin
                                paquete = imp.load_module( nombre_plugin, f, filename, description )
                                self.plugins.append( paquete.classFactory( self.qgisInterface ) )
                            except Exception, e:
                                print '    ERROR!:',e
                            else:
                                self.plugins[ -1 ].initGui()
                                print '    OK!'
        else:
            print "Plugins folder not found."

    def crearMenuPlugins( self ):
        """ Create the Plugins menu on menu bar """
        self.pyQgisApp.menuPlugins = QMenu( "&Plugins", self.pyQgisApp.menuBar )
        actions = self.pyQgisApp.menuBar.actions()
        lastAction = actions[ len( actions ) - 1 ]
        self.pyQgisApp.menuBar.insertMenu( lastAction, self.pyQgisApp.menuPlugins )